// Global Variables
var levels;
//var sounds = new Sounds();
var cube = [];
var floorTile = [];
var scale = 20;
var tileAnimation = false;
var currentLevel = 1;

var rows = 9;
var cols = 9;

var leftUpPressed = false;
var rightUpPressed = false;
var leftDownPressed = false;
var rightDownPressed = false;
var spacePressed = false;
var spaceHeld = false;
var buttonReleased = false;

var cubesAnimating = false;

var levelChanging = false;
var resetCubes = false;
var fadeCubes = true;

var moveCounter = 0;
var levelMoveCounter = 0;

var resetMenu = false;

var grd; // background gradient

// touch control variables
var touchStart = new PVector(0,0);
var touchCurrent = new PVector(0,0);
var touchTapStart;
var touchTapTime = 300;
var touchRangeInner = 25;
var touchRangeOuter = 35;

// mobile device
var isOnMobile = false;
var mobileTested = false;


// setup
var setUp = function(){

	// make sure there are no cubes
	if(cube.length > 0){
		cube.splice(0,cube.length);
	}

	// et all the starting values
	scale = 20;
	tileAnimation = false;
	currentLevel = 1;

	rows = 9;
	cols = 9;

	leftUpPressed = false;
	rightUpPressed = false;
	leftDownPressed = false;
	rightDownPressed = false;
	spacePressed = false;
	spaceHeld = false;
	buttonReleased = false;

	cubesAnimating = false;

	levelChanging = false;
	resetCubes = false;
	fadeCubes = true;

	moveCounter = 0;
	levelMoveCounter = 0;
	
	resetMenu = false;


	levels = new Level();
	
	for(var i = 0; i < rows; i++){
		floorTile[i] = [];
		for(var j = 0; j < cols; j++){
			floorTile[i][j] = new FloorTile();
			floorTile[i][j].constructor(i,j);
		}
	}
	
	// load and save states
	try{
		if(typeof(Storage) !== "undefined"){
				
			if(localStorage.storedLevel && !isNaN(localStorage.storedLevel) ){
				currentLevel = Number(localStorage.storedLevel);
				console.log("loading level " + localStorage.storedLevel);
				localStorage.storedLevel = Number(currentLevel);
			} 
			else{
				currentLevel = 1;
				console.log("creating level " + currentLevel);
				localStorage.storedLevel = Number(currentLevel);
			}
		
		
			if(localStorage.storedMoves && !isNaN(localStorage.storedMoves) ){
				moveCounter = Number(localStorage.storedMoves);
				console.log("loading moves " + localStorage.storedMoves);
				localStorage.storedMoves = Number(moveCounter);
			} 
			else{
				moveCounter = 0;
				console.log("moveCounter " + moveCounter);
				localStorage.storedMoves = Number(moveCounter);
			}
		}
	}
	catch(err){
		console.log(err);
		currentLevel = 1;
		moveCounter = 0;
	}
	
	
	levels.setLevel(currentLevel);
	
	
	// Create gradient
    //var grd = context.createLinearGradient(canvas.width/2, canvas.height, canvas.width/2, 0.000);
    grd = context.createLinearGradient(0.000, canvas.height/2, canvas.width, canvas.height/2);
      
    // Add colors
    grd.addColorStop(0.000, 'rgba(255, 233, 0, 1.000)');
    grd.addColorStop(0.150, 'rgba(255, 255, 0, 1.000)');
    grd.addColorStop(0.850, 'rgba(255, 255, 0, 1.000)');
    grd.addColorStop(1.000, 'rgba(255, 233, 0, 1.000)');
    
    // 
    
	
}

// main game loops
var updateGame = function(){
	//
	if(!mobileTested){
		if( isMobile.any() ){
			isOnMobile = true;
		}
		mobileTested = true;
		console.log(isOnMobile);
	}


	// update the tiles
	tileAnimation = false;
	for(var i = 0; i < rows; i++){
		for(var j = 0; j < cols; j++){
			floorTile[i][j].update();
			if(floorTile[i][j].getIsAnimating()){
				tileAnimation = true;
			}
		}
	}
	
	
	// PAUSE / MAKE SURE CUBES ARE ON TOP
	///////////////////////////////////////////////////////////////////////
	if(resetMenu){
		//console.log("reset menu");
		levels.setDisplayReset(true);
	}
	else if(resetCubes){
		//console.log("resetCubes");
		// test for cube animation
		cubesAnimating = false;
		for(var i = 0; i < cube.length; i++){
			if(cube[i].getIsAnimating()){
				// do not send move command if animating other cubes
				cubesAnimating = true;
			}
		}
		if(cubesAnimating){
			for(var i = 0; i < cube.length; i++){
				cube[i].setIsStatic();
				cube[i].updateRiseFall();
				cube[i].updateHeight();
			}
		}
		else{
			for(var i = 0; i < cube.length; i++){
				cube[i].setTargetTH(160); // raise cubes before animating
			}
		
			fadeCubes = true;
			resetCubes = false;
			    
			currentLevel++;
			moveCounter += levelMoveCounter;
			levelMoveCounter = 0;
			levels.setLevel(currentLevel);
			saveStates();	
		}
	}	
	
	// ANIMATING BETWEEN LEVELS
	///////////////////////////////////////////////////////////////////////
	else if(tileAnimation){
		//console.log("tileAnimation");
		for(var i = 0; i < cube.length; i++){
			cube[i].updateRiseFall();
		}
	}
	// FADE CUBES BACK TO RED
	///////////////////////////////////////////////////////////////////////
	else if(fadeCubes){
		//console.log("fadeCubes");
		for(var i = 0; i < cube.length; i++){
			cube[i].setIsOnTarget(false);
			cube[i].setFade(true);
		}
		
		if(cube[0].getFade()){
			fadeCubes = false;
		}
	}
	
	// IN GAME LEVEL
	///////////////////////////////////////////////////////////////////////
	else if(!tileAnimation){
		//console.log("!tileAnimation");
		// set cubes to can move
		for(var i = 0; i < cube.length; i++){
			cube[i].setCanMove();
		}
		
		// send tile heights to cube
		for(var i = 0; i < cube.length; i++){
			// update cubes current height position
			// update neighbours, i.e. cannot move due to walls.
			cube[i].setTileHeights(floorTile);
		}
		
		// test for cube animation
		cubesAnimating = false;
		for(var i = 0; i < cube.length; i++){
			if(cube[i].getIsAnimating()){
				// do not send move command if animating other cubes
				cubesAnimating = true;
			}
		}
	
		// check walls
		for(var i = 0; i < cube.length; i++){
			cube[i].checkEdgeOfTiles();
		}
	
		// send other cubes info to cubes
		for(var l = 0; l < cube.length; l++){ // need to loop round to make sure chains of cubes hit each other
			for(var i = 0; i < cube.length; i++){
				for(var j = 0; j < cube.length; j++){
					if(i != j){
						for(var k = 0; k < 4; k++){
							cube[i].checkOthers(cube[j].getXPosition(),cube[j].getYPosition(),cube[j].getCanMoveDir(k),k); // 0 is direction
						}
					}
				}
			}
		}
		
		// undo move if button released
		if(buttonReleased){
			for(var i = 0; i < cube.length; i++){
				cube[i].setRotBack(true);
			}
			buttonReleased = false;
		}
	
		// send move commands
		if(!cubesAnimating){
			for(var i = 0; i < cube.length; i++){	
				if(leftUpPressed){
					cube[i].move(0);
				}
				if(rightUpPressed){
					cube[i].move(1);
				}
				if(leftDownPressed){
					cube[i].move(2);
				}
				if(rightDownPressed){
					cube[i].move(3);
				}
				if(spacePressed){
					cube[i].move(4);
				}
			}
		}
		spacePressed = false;	
		
		// update cubes
		for(var i = 0; i < cube.length; i++){
			cube[i].updateRiseFall();
			cube[i].updateRotation();
			cube[i].updateHeight();
		}
	
	
		// tell cube if it is on target or not
		if(!cubesAnimating){
			for(var i = 0; i < cube.length; i++){
				cube[i].setIsOnTarget(false);
			}
			
	
			for(var i = 0; i < rows; i++){
				for(var j = 0; j < cols; j++){
					if(floorTile[i][j].getIsTarget()){
						for(var k = 0; k < cube.length; k++){
							cube[k].target(i,j);
						}
					}
				}
			}
		}
		
		
		// test for cubes on targets
		var noCubesOnTarget = 0;
		for(var i = 0; i < cube.length; i++){
			if(cube[i].getIsOnTarget()){
				noCubesOnTarget++;
			}
		}
		
		if(noCubesOnTarget == cube.length){
			resetCubes = true;
			
			for(var i = 0; i < cube.length; i++){
				cube[i].rightWayUp();
			}
		}
	}
	
	//////////////////////////////
	// check for movement
	var movement = false
	for(var i = 0; i < cube.length; i++){
		if(cube[i].getHasMoved()){
			movement = true;
		}
	}
	if(movement){
		levelMoveCounter++;
		//moveCounter++;
		//console.log("Moved " + moveCounter);
	}
		
	fps.update();
}

var drawGame = function(){
      
    // Fill with gradient
    context.fillStyle = grd;

	// refresh background
	//context.fillStyle = 'yellow';
	context.fillRect(0,0,canvas.width,canvas.height);
	
	
	var alpha;
	if(cube.length > 0){
		alpha = (cube[0].getCubeHeight() + scale) / (2*scale);
	}
	else{
		alpha = 0;
	}
	
	// draw floor tiles
	for(var i = rows-1; i >= 0; i--){
		for(var j = 0; j < cols; j++){
			floorTile[i][j].calcGrd(alpha);
			floorTile[i][j].display();
		}
	}
	
	// draw floor tiles
	for(var i = rows-1; i >= 0; i--){
		for(var j = 0; j < cols; j++){
			floorTile[i][j].displayTarget();
		}
	}
	
	/// alternative display order (works if rows and cols are the same value?)
	// start 0: (rows, 0)					
	//		 1: (rows-1, 0) (rows, 1)
	//		 2: (rows-2, 0) (rows-1,1) (rows, 2)
	//		 3: (rows-3, 0) (rows-2,1) (rows-1,2) (rows, 3)
	
	// start 0: (rows-0, 0)					
	//		 1: (rows-1, 0) (rows, 1)
	//		 2: (rows-2, 0) (rows-1,1) (rows, 2)
	//		 3: (rows-3, 0) (rows-2,1) (rows-1,2) (rows, 3)
	
	
	// Draw the cubes
	if(cube.length > 0){
	
		var displayOrder = [];
		var altMeth = false;
	
	
		for(var i = 0; i < cube.length; i++){
			if(cube[i].getAniLeftDown() || cube[i].getAniRightUp()){
				altMeth = true;
			}
		}
	
		if(!altMeth){
			// first method
			var ref = 0;
			for(var i = 0; i < cols; i++){
				for(var j = rows-1; j >= 0; j--){	
					for(var k = 0; k < cube.length; k++){
						if(cube[k].getXPosition() == i && cube[k].getYPosition() == j){
							displayOrder[ref++] = k;   
						} 
					}
				}
			}
		
			// draw cubes with first method
			if(cube[0].getCubeHeight() > 0){ 
				for(var i = cube.length-1; i >= 0; i--){
					cube[displayOrder[i]].display();
				}
			}
			else{
				for(var i = 0; i < cube.length; i++){
					cube[displayOrder[i]].display();
				}
			}				
		}
		else{
			// second method
			ref = 0;
			for(var i = 0; i < rows*2; i++){
				for(var j = 0; j < i+1; j++){
					if((rows -(i-j)) > 0 && j < cols){	
						for(var k = 0; k < cube.length; k++){
							if(cube[k].getXPosition() == (rows -(i-j))-1 && cube[k].getYPosition() == j){
								displayOrder[ref++] = k; 
							} 	
						}
					}	
				}
			}
		
			// draw cubes with second method
			if(cube[0].getCubeHeight() < 0){ 
				for(var i = cube.length-1; i >= 0; i--){
					cube[displayOrder[i]].display();
				}
			}
			else{
				for(var i = 0; i < cube.length; i++){
					cube[displayOrder[i]].display();
				}
			}
		}
	}
	
	levels.display();
	
	//fps.draw();
}

/////////////////////////
// mouse input functions
function getMousePos(canvas, evt) {
    var rect = canvas.getBoundingClientRect();
    return {
        x: evt.clientX - rect.left,
        y: evt.clientY - rect.top
    };
}

canvas.addEventListener('mousemove', function(evt) {
    var mousePos = getMousePos(canvas, evt);
	mouseX = mousePos.x;
	mouseY = mousePos.y;
}, false);

///////////////////////////////////////////////////////////////////
// touch events
canvas.addEventListener("touchstart", function(){
	//console.log("touchstart");	
	
	if (!e)
        var e = event;
    e.preventDefault();
    //canX = e.targetTouches[0].pageX - canvas.offsetLeft;
    //canY = e.targetTouches[0].pageY - canvas.offsetTop;
    
    // keep above here
    
    touchTapStart = new Date().getTime();
    
    touchStart.setXAndY(e.targetTouches[0].pageX - canvas.offsetLeft,e.targetTouches[0].pageY - canvas.offsetTop);
    
	
}, false);

canvas.addEventListener("touchmove", function(e){
	//console.log("touchmove");	
	
	if (!e)
        var e = event;
    e.preventDefault();
    //canX = e.targetTouches[0].pageX - canvas.offsetLeft;
    //canY = e.targetTouches[0].pageY - canvas.offsetTop;
    
    
    // keep above here
	
	leftUpPressed = false;
	rightUpPressed = false;
	leftDownPressed = false;
	rightDownPressed = false;
    
    touchCurrent.setXAndY(e.targetTouches[0].pageX - canvas.offsetLeft,e.targetTouches[0].pageY - canvas.offsetTop);
    
    if(dist(touchStart,touchCurrent) > touchRangeOuter){
    	//console.log("too big");
    	var fraction = (dist(touchStart,touchCurrent) - touchRangeOuter) / dist(touchStart,touchCurrent);
    	touchStart.movePointBetween(touchStart,touchCurrent,fraction);
    }
    
    
    var angle = angleBetween(touchStart,touchCurrent);
    
    if(!levelChanging){ 
    	if(dist(touchStart,touchCurrent) > touchRangeInner){
			if(angle <= 0 && angle >= -Math.PI/2){
				//console.log("up right");
				rightUpPressed = true;
			}
			else if(angle <= -Math.PI/2 && angle >= -Math.PI){
				//console.log("up left");
				leftUpPressed = true;
			}
			else if(angle <= Math.PI/2 && angle >= 0){
				//console.log("down right");
				rightDownPressed = true;
			}	
			else if(angle <= Math.PI && angle >= Math.PI/2){
				//console.log("down left");
				leftDownPressed = true;
			}
		}
		else{
			//console.log("don't move");
			leftUpPressed = false;
			rightUpPressed = false;
			leftDownPressed = false;
			rightDownPressed = false;
		}
    }
	
}, false);


canvas.addEventListener("touchend", function(){
	
	//console.log("touchend");
	leftUpPressed = false;
	rightUpPressed = false;
	leftDownPressed = false;
	rightDownPressed = false;
	
	touchTapEnd = new Date().getTime();
	console.log(touchTapEnd + " " + touchTapStart);
	
	if(touchTapEnd < touchTapStart + touchTapTime){
    	spacePressed = true;
    }

}, false);

document.body.addEventListener("touchcancel", function(){
	
	//console.log("touchcancel");
	leftUpPressed = false;
	rightUpPressed = false;
	leftDownPressed = false;
	rightDownPressed = false;
	
	touchTapEnd = new Date().getTime();
	console.log(touchTapEnd + " " + touchTapStart);
	
	if(touchTapEnd < touchTapStart + touchTapTime){
    	spacePressed = true;
    }

}, false);

///////////////////////////////////////////////////////////////////
// KEYBOARD EVENTS
document.addEventListener('keydown', function(event) {
    //console.log("keydown");
    
	leftUpPressed = false;
	rightUpPressed = false;
	leftDownPressed = false;
	rightDownPressed = false;
	spacePressed = false;
	
	if(!levelChanging){
		if(event.keyCode == 87 || event.keyCode == 38) {
			//console.log('UP/LEFT');
			leftUpPressed = true;
			event.preventDefault();
		}
		else if(event.keyCode == 68 || event.keyCode == 39) {
			//console.log('UP/RIGHT');
			rightUpPressed = true;
			event.preventDefault();
		}
		else if(event.keyCode == 65 || event.keyCode == 37) {
			//console.log('DOWN/LEFT');
			leftDownPressed = true;
			event.preventDefault();
		}
		else if(event.keyCode == 83 || event.keyCode == 40) {
			//console.log('DOWN/RIGHT');
			rightDownPressed = true;
			event.preventDefault();
		}
		else if(event.keyCode == 16  || event.keyCode == 32) {
			//console.log('FLIP');
			if(!spacePressed && !spaceHeld){
				spacePressed = true;
				spaceHeld = true;
			}
			event.preventDefault();
		}	
	}
	
	// skip levels <------ remove from final build
	/*
	if(event.keyCode == 48){
		levels.next(1);
	}
	if(event.keyCode == 189){
		levels.next(5);
	}
	if(event.keyCode == 187){
		levels.next(10);
	}
	if(event.keyCode == 57){
		levels.next(-2);
		resetCubes = true;
		
		for(var i = 0; i < cube.length; i++){
			cube[i].setHeight();
		}
	}
	*/
	
	// reset levels
	if(event.keyCode == 82){
		levels.next(-1);
		resetCubes = true;
		levelMoveCounter = 0;
		
		for(var i = 0; i < cube.length; i++){
			cube[i].setHeight();
		}
	}
	
	
	// reset menu
	if(resetMenu){
		if(event.keyCode == 89){ // yes
			resetMenu = false;
			deleteStates();
			resetCubes = true;
			currentLevel = 0;
			levels.setDisplayReset(false);
			setUp();
		}
	
		if(event.keyCode == 78){ // no
			resetMenu = false;
			levels.setDisplayReset(false);
		}
	}
	
	if(event.keyCode == 13){
		resetMenu = true;
	}
		
	//console.log(event.keyCode); // a is 65 -> z is 90
});

document.addEventListener('keyup', function(event) {
	if(event.keyCode == 87 || event.keyCode == 38) {
		//console.log('f UP/LEFT');
		leftUpPressed = false;
		buttonReleased = true;
	}
	else if(event.keyCode == 68 || event.keyCode == 39) {
		//console.log('f UP/RIGHT');
		rightUpPressed = false;
		buttonReleased = true;
	}
	else if(event.keyCode == 65 || event.keyCode == 37) {
		//console.log('f DOWN/LEFT');
		leftDownPressed = false;
		buttonReleased = true;
	}
	else if(event.keyCode == 83  || event.keyCode == 40) {
		//console.log('f DOWN/RIGHT');
		rightDownPressed = false;
		buttonReleased = true;
	}
	else if(event.keyCode == 16  || event.keyCode == 32) {
		//console.log('f FLIP');
		spaceHeld = false;
	}
	
	
});


// do not adjust below here

// from http://www.playmycode.com/blog/2011/08/building-a-game-mainloop-in-javascript/

var mainloop = function() {
    updateGame();
    drawGame(); 
};
	
var animFrame = window.requestAnimationFrame ||
    window.webkitRequestAnimationFrame ||
    window.mozRequestAnimationFrame    ||
    window.oRequestAnimationFrame      ||
    window.msRequestAnimationFrame     ||
    null ;

if ( animFrame !== null ) {
    //var canvas = document.getElementById('myCanvas');
	//var context = canvas.getContext('2d'); // <--------- I have added this

    var recursiveAnim = function() {
        mainloop();
        animFrame( recursiveAnim, canvas );
    };

	setUp(); // <----- I have added this
	
	// start the mainloop
    animFrame( recursiveAnim, canvas );
} else {
        var ONE_FRAME_TIME = 1000.0 / 60.0 ;
        setInterval( mainloop, ONE_FRAME_TIME );
}

var fps = {
	current: 0,
	last: 0,
	lastUpdated: Date.now(),
	delta: 0, // <--- I added this
	previous: Date.now(), // <--- I added this
	
	draw: function(){
		context.fillStyle = 'white';
		context.fillRect(0,0,50,25);
		context.font = '12pt Arial';
		context.fillStyle = 'black';
		context.textBaseline = 'top';
		context.textAlign = "left";
		context.fillText(fps.last + 'fps', 5, 5);
	},
	update: function(){
		fps.current++;
		
		fps.delta = (Date.now() - fps.previous) / (1000/60); // <---- I added this
		fps.previous = Date.now();
		
		if(fps.delta > 5.0){
			fps.delta = 5.0;
		}
				
		if(Date.now() - fps.lastUpdated >= 1000){		
			fps.last = fps.current;
			fps.current = 0;
			fps.lastUpdated = Date.now();
		}
	}
}

//////////////
// SAVING STATES
var saveStates = function(){	
	try{
		if(typeof(Storage) !== "undefined") {
			// levels
			if (localStorage.storedLevel) {
				localStorage.storedLevel = Number(currentLevel);
				console.log("Save " + localStorage.storedLevel);
			}
			else{
				console.log("Save new");
				localStorage.storedLevel = Number(currentLevel);
			}
		
			// moves
			if (localStorage.storedMoves) {
				localStorage.storedMoves = Number(moveCounter);
				console.log("Save moves " + localStorage.storedMoves);
			}
			else{
				console.log("Save new moves");
				localStorage.storedMoves = Number(moveCounter);
			}
		} 
    }
    catch(err){
    	console.log(err);
    	console.log("did not save");
    }
}

var loadStates = function(){
	if(typeof(Storage) !== "undefined"){
		if(localStorage.storedLevel){
			currentLevel = Number(localStorage.storedLevel);
		} 
		else{
			currentLevel = 1;
			localStorage.storedLevel = Number(currentLevel);
		}
	}
}

var deleteStates = function(){
	try{
		if(typeof(Storage) !== "undefined"){
			if(localStorage.storedLevel){
				localStorage.removeItem("storedLevel");
				console.log("delete storedLevel");
			}
		
			if(localStorage.storedMoves){
				localStorage.removeItem("storedMoves");
				console.log("delete storedMoves");
			}
		}
	}
	catch(err){
		console.log(err);
		console.log("can not delete");
	}
}

///////////////////////////////////////////////////////
/// detecting mobile device
var isMobile = {
    Android: function() {
        return navigator.userAgent.match(/Android/i);
    },
    BlackBerry: function() {
        return navigator.userAgent.match(/BlackBerry/i);
    },
    iOS: function() {
        return navigator.userAgent.match(/iPhone|iPad|iPod/i);
    },
    Opera: function() {
        return navigator.userAgent.match(/Opera Mini/i);
    },
    Windows: function() {
        return navigator.userAgent.match(/IEMobile/i);
    },
    any: function() {
        return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Opera() || isMobile.Windows());
    }
};
	
// do not adjust before here